from .helpers import *

class TestClientWorkspaces(ClientTestCase):

    def test_workspaces_get_workspaces(self):
        res = { "data": [{ "id": 1337, "name": "My Favorite Workspace" }]}
        responses.add(GET, 'http://app/workspaces', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.workspaces.get_workspaces(), res['data'])

    def test_workspaces_get_workspace(self):
        res = { "data": [{ "id": 1337, "name": "My Favorite Workspace" }]}
        responses.add(GET, 'http://app/workspaces/222', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.workspaces.get_workspace('222'), res['data'])

    def test_workspaces_update_workspace(self):
        req = { "data": { "name": "Everyone's Favorite Workspace" } }
        res = { "data": { "id": 1337, "name": "Everyone's Favorite Workspace" } }
        responses.add(PUT, 'http://app/workspaces/1337', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.workspaces.update_workspace('1337', req['data']), res['data'])
        self.assertEqual(json.loads(responses.calls[0].request.body), req)



from .helpers import *

class TestClientCustomFields(ClientTestCase):
    def test_custom_fields_get_custom_field_text(self):
        res = {
            "data": [
                {
                    "id": 134679,
                    "created_at": "2016-07-11T14:29:23.249Z",
                    "name": "Owner",
                    "type": "text"
                }
            ]
        }
        responses.add(GET, 'http://app/custom_fields/134679', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.custom_fields.get_custom_field('134679'), res['data'])

    def test_custom_fields_get_custom_field_enum(self):
        res = {
            "data": [
                {
                    "id": 124578,
                    "created_at": "2016-07-11T14:29:23.249Z",
                    "name": "Priority",
                    "type": "enum",
                    "enum_options": [
                        {
                            "id": 789,
                            "name": "Low",
                            "enabled": true,
                            "color": "blue"
                        },
                        {
                            "id": 208,
                            "name": "Medium",
                            "enabled": false,
                            "color": "yellow"
                        },
                        {
                            "id": 439,
                            "name": "High",
                            "enabled": true,
                            "color": "red"
                        }
                    ]
                }
            ]
        }
        responses.add(GET, 'http://app/custom_fields/124578', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.custom_fields.get_custom_field('124578'), res['data'])

    def test_custom_fields_get_custom_field_number(self):
        res = {
            "data": [
                {
                    "id": 938271,
                    "created_at": "2016-07-11T14:29:23.249Z",
                    "name": "Price",
                    "type": "number",
                    "precision": 2
                }
            ]
        }
        responses.add(GET, 'http://app/custom_fields/938271', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.custom_fields.get_custom_field('938271'), res['data'])



from .helpers import *

class TestClientTasks(ClientTestCase):

    def test_tasks_create_task(self):
        req = {
            "data": {
                "assignee": 1235,
                "followers": [5678],
                "name": "Hello, world.",
                "notes": "How are you today?",
                "workspace": 14916
            }
        }
        res = {
            "data": {
                "assignee": { "id": 1235, "name": "Tim Bizarro" },
                "assignee_status": "inbox",
                "completed": false,
                "completed_at": null,
                "created_at": "2012-02-22T02:06:58.158Z",
                "due_on": null,
                "followers": [{ "id": 5678, "name": "Greg Sanchez" } ],
                "id": 1001,
                "modified_at": "2012-02-22T02:06:58.158Z",
                "name": "Hello, world!",
                "notes": "How are you today?",
                "parent": null,
                "projects": [{ "id": 14641, "name": "Cat Stuff" }],
                "workspace": { "id": 14916, "name": "My Favorite Workspace" }
            }
        }
        responses.add(POST, 'http://app/tasks', status=201, body=json.dumps(res), match_querystring=True)

        self.assertEqual(self.client.tasks.create_task(req['data']), res['data'])
        self.assertEqual(json.loads(responses.calls[0].request.body), req)

    def test_tasks_get_task(self):
        res = {
            "data": {
                "assignee": { "id": 1234, "name": "Tim Bizarro" },
                "created_at": "2012-02-22T02:06:58.158Z"
            }
        }
        responses.add(GET, 'http://app/tasks/1001', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.tasks.get_task('1001'), res['data'])

    def test_tasks_get_tasks_for_project(self):
        res = {
            "data": [
                { "id": 2001, "name": "Catnip" },
                { "id": 2002, "name": "Kitty litter" }
            ]
        }
        responses.add(GET, 'http://app/projects/1331/tasks', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.tasks.get_tasks_for_project('1331'), res['data'])

    def test_tasks_update_task(self):
        req = { "data": { "assignee": "me" } }
        res = {
            "data": {
                "assignee": { "id": 1234, "name": "Tim Bizarro" },
                "id": 1001
            }
        }
        responses.add(PUT, 'http://app/tasks/1001', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.tasks.update_task('1001', req['data']), res['data'])
        self.assertEqual(json.loads(responses.calls[0].request.body), req)

    def test_tasks_delete_task(self):
        res = { "data": {} }
        responses.add(DELETE, 'http://app/tasks/1001', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.tasks.delete_task('1001'), res['data'])

    def test_tasks_get_tasks(self):
        res = {
            "data": [
                { "id": 1248, "name": "Buy catnip" },
                { "id": 24816, "name": "Reflect on role of kittens in society" }
            ]
        }
        responses.add(GET, 'http://app/tasks?workspace=14916&assignee=me', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.tasks.get_tasks({ 'workspace': 14916, 'assignee': 'me' }), res['data'])

    def test_tasks_add_subtask(self):
        req = {
            "data": {
                "assignee": 1235,
                "followers": [5678],
                "name": "Tell Luke",
                "notes": "He's going to be upset."
            }
        }
        res = {
            "data": {
                "assignee": { "id": 1235, "name": "Darth Vader" },
                "assignee_status": "inbox",
                "completed": false,
                "completed_at": null,
                "created_at": "2012-02-22T02:06:58.158Z",
                "due_on": null,
                "followers": [{ "id": 5678, "name": "Emperor Palpatine" } ],
                "id": 1001,
                "modified_at": "2012-02-22T02:06:58.158Z",
                "name": "Tell Luke",
                "notes": "He's going to be upset.",
                "parent": { "id": 2272, "name": "Tell kids I am their father." },
                "projects": [],
                "workspace": { "id": 14916, "name": "Star Wars" }
            }
        }
        responses.add(POST, 'http://app/tasks/2272/subtasks', status=201, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.tasks.add_subtask('2272', req['data']), res['data'])
        self.assertEqual(json.loads(responses.calls[0].request.body), req)

    def test_tasks_subtasks(self):
        res = {
            "data": [
                { "id": 5005, "name": "Steal Underwear" },
                { "id": 6709, "name": "???" },
                { "id": 9812, "name": "Profit" }
            ]
        }
        responses.add(GET, 'http://app/tasks/7331/subtasks', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.tasks.subtasks(7331), res['data'])

    def test_tasks_set_parent(self):
        req = { "data": { "parent": 1331 } }
        res = {
            "data": {
                "id": 2272,
                "name": "Tell Luke",
                "parent": [{ "id": 1331, "name": "Tell kids I am their father" }]
            }
        }
        responses.add(POST, 'http://app/tasks/2272/setParent', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.tasks.set_parent('2272', req['data']), res['data'])
        self.assertEqual(json.loads(responses.calls[0].request.body), req)

    def test_tasks_projects(self):
        res = {
            "data": [
                { "id": 1331, "name": "Things To Buy" },
                { "id": 14641, "name": "Cat Stuff" }
            ]
        }
        responses.add(GET, 'http://app/tasks/1001/projects', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.tasks.projects('1001'), res['data'])

    def test_tasks_add_project(self):
        req = { "data": { "project": 14641 } }
        res = { "data": {} }
        responses.add(POST, 'http://app/tasks/1001/addProject', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.tasks.add_project('1001', req['data']), res['data'])
        self.assertEqual(json.loads(responses.calls[0].request.body), req)

    def test_tasks_remove_project(self):
        req = { "data": { "project": 14641 } }
        res = { "data": {} }
        responses.add(POST, 'http://app/tasks/1001/removeProject', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.tasks.remove_project('1001', req['data']), res['data'])
        self.assertEqual(json.loads(responses.calls[0].request.body), req)

    def test_tasks_tags(self):
        res = {
            "data": [
                { "id": 1331, "name": "orange" },
                { "id": 1771, "name": "fluffy" }
            ]
        }
        responses.add(GET, 'http://app/tasks/1001/tags', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.tasks.tags('1001'), res['data'])

    def test_tasks_1001_addTag(self):
        req = { "data": { "tag": 1771 } }
        res = { "data": {} }
        responses.add(POST, 'http://app/tasks/1001/addTag', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.tasks.add_tag('1001', req['data']), res['data'])
        self.assertEqual(json.loads(responses.calls[0].request.body), req)

    def test_tasks_removeTag(self):
        req = { "data": { "tag": 1771 } }
        res = { "data": {} }
        responses.add(POST, 'http://app/tasks/1001/removeTag', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.tasks.remove_tag('1001', req['data']), res['data'])
        self.assertEqual(json.loads(responses.calls[0].request.body), req)

    def test_tasks_add_followers(self):
        req = { "data": { "followers": [1235] } }
        res = {
            "data": {
                "followers": [{ "id": 1235, "name": "Darth Vader" }],
                "id": 1001
            }
        }
        responses.add(POST, 'http://app/tasks/1001/addFollowers', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.tasks.add_followers('1001', req['data']), res['data'])
        self.assertEqual(json.loads(responses.calls[0].request.body), req)

    def test_tasks_remove_followers(self):
        req = { "data": { "followers": [1235] } }
        res = { "data": { "followers": [], "id": 1001 } }
        responses.add(POST, 'http://app/tasks/1001/removeFollowers', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.tasks.remove_followers('1001', req['data']), res['data'])
        self.assertEqual(json.loads(responses.calls[0].request.body), req)

    def test_tasks_get_tasks_for_tag(self):
        res = {
            "data": [
                { "id": 2001, "name": "Catnip" },
                { "id": 2002, "name": "Kitty litter" }
            ]
        }
        responses.add(GET, 'http://app/tags/1331/tasks', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.tasks.get_tasks_for_tag('1331'), res['data'])

    def test_tasks_custom_field_data(self):
        res = {
            "data": {
                "id": 1001,
                "name": "Hello, world!",
                "completed": false,
                "custom_fields": [
                    {
                        "id": 124578,
                        "name": "Priority",
                        "type": "enum",
                        "enum_value": {
                          "id": 789,
                          "name": "Low",
                          "enabled": true,
                          "color": "blue"
                        }
                    }
                ]
            }
        }
        responses.add(GET, 'http://app/tasks/1001', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.tasks.get_task('1001'), res['data'])



from .helpers import *

class TestClientEvents(ClientTestCase):

    def test_events_get(self):
        res = {
            "data": [
                {
                    "action": "changed",
                    "created_at": "2013-08-21T18:20:37.972Z",
                    "parent": null,
                    "resource": { "id": 1337, "name": "My Task" },
                    "type": "task",
                    "user": { "id": 1123, "name": "Tom Bizarro" }
                },
                {
                    "action": "changed",
                    "created_at": "2013-08-21T18:22:45.421Z",
                    "parent": null,
                    "resource": { "id": 1338, "name": "My Other Task" },
                    "type": "task",
                    "user": { "id": 1428, "name": "Greg Sanchez" }
                }
            ],
            "sync": "edfc0896b370b7a479886d316131bf5c:0"
        }
        responses.add(GET, 'http://app/events?resource=14321&sync=de4774f6915eae04714ca93bb2f5ee81%3A3', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.events.get({ 'resource': 14321, 'sync': 'de4774f6915eae04714ca93bb2f5ee81:3' }), res)

    def test_project_events_get_invalid_token(self):
        res = {
            "message": "Sync token invalid or too old. If you are attemping to keep resources in sync, you must re-fetch the full dataset for this query now.",
            "sync": "edfc0896b370b7a479886d316131bf5c:0"
        }
        responses.add(GET, 'http://app/events?resource=3214312&sync=de4774f6915eae04714ca93bb2f5ee81%3A3', status=412, body=json.dumps(res), match_querystring=True)
        # responses.add(GET, 'http://app/project/3214312/events?sync=de4774f6915eae04714ca93bb2f5ee81%3A3', status=412, body=json.dumps(res), match_querystring=True)
        self.assertRaises(asana.error.InvalidTokenError, self.client.events.get, ({ 'resource': 3214312, 'sync': 'de4774f6915eae04714ca93bb2f5ee81:3' }))
        # self.assertEqual(self.client.events.get({ 'resource': 3214312, 'sync': 'de4774f6915eae04714ca93bb2f5ee81:3' }), res)

    def test_project_events_get_next(self):
        responses.add(GET, 'http://app/events?resource=1', status=412, body=json.dumps({ "sync": "1" }), match_querystring=True)
        responses.add(GET, 'http://app/events?resource=1&sync=1', status=200, body=json.dumps({ "data": [], "sync": "2" }), match_querystring=True)
        responses.add(GET, 'http://app/events?resource=1&sync=2', status=200, body=json.dumps({ "data": [{}, {}, {}], "sync": "3" }), match_querystring=True)

        self.assertEqual(len(self.client.events.get_next({ "resource": "1" })[0]), 3)

    def test_project_events_get_next_unknown_object(self):
        responses.add(GET, 'http://app/events?resource=1', status=400, body=json.dumps({ "sync": "1" }), match_querystring=True)

        self.assertRaises(asana.error.InvalidRequestError, self.client.events.get_next, ({ "resource": 1 }))

    def test_project_events_get_next_invalid_token(self):
        responses.add(GET, 'http://app/events?resource=1&sync=invalid', status=412, body=json.dumps({ "sync": "1" }), match_querystring=True)

        self.assertRaises(asana.error.InvalidTokenError, self.client.events.get_next, ({ "resource": 1, "sync": "invalid" }))

    def test_project_events_get_iterator(self):
        responses.add(GET, 'http://app/events?resource=1', status=412, body=json.dumps({ "sync": "1" }), match_querystring=True)
        responses.add(GET, 'http://app/events?resource=1&sync=1', status=200, body=json.dumps({ "data": [], "sync": "2" }), match_querystring=True)
        responses.add(GET, 'http://app/events?resource=1&sync=2', status=200, body=json.dumps({ "data": ['a', 'b'], "sync": "3" }), match_querystring=True)
        responses.add(GET, 'http://app/events?resource=1&sync=3', status=200, body=json.dumps({ "data": ['c'], "sync": "4" }), match_querystring=True)

        iterator = self.client.events.get_iterator({ "resource": "1" })
        self.assertEqual(next(iterator), 'a')
        self.assertEqual(next(iterator), 'b')
        self.assertEqual(next(iterator), 'c')

    def test_project_events_get_iterator_unknown_object(self):
        responses.add(GET, 'http://app/events?resource=1', status=400, body=json.dumps({ "sync": "1" }), match_querystring=True)

        iterator = self.client.events.get_iterator({ "resource": 1 })
        self.assertRaises(asana.error.InvalidRequestError, next, (iterator))

    def test_project_events_get_iterator_invalid_token(self):
        responses.add(GET, 'http://app/events?resource=1&sync=invalid', status=412, body=json.dumps({ "sync": "1" }), match_querystring=True)

        iterator = self.client.events.get_iterator({ "resource": 1, "sync": "invalid" })
        self.assertRaises(asana.error.InvalidTokenError, next, (iterator))



# coding=utf-8
class _Projects:

    def __init__(self, client=None):
        self.client = client

    def add_custom_field_setting_for_project(self, project_gid, params=None, **options):
        """Add a custom field to a project
        :param str project_gid: (required) Globally unique identifier for the project.
        :param Object params: Parameters for the request
        :param **options
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/projects/{project_gid}/addCustomFieldSetting".replace("{project_gid}", project_gid)
        return self.client.post(path, params, **options)

    def add_followers_for_project(self, project_gid, params=None, **options):
        """Add followers to a project
        :param str project_gid: (required) Globally unique identifier for the project.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/projects/{project_gid}/addFollowers".replace("{project_gid}", project_gid)
        return self.client.post(path, params, **options)

    def add_members_for_project(self, project_gid, params=None, **options):
        """Add users to a project
        :param str project_gid: (required) Globally unique identifier for the project.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/projects/{project_gid}/addMembers".replace("{project_gid}", project_gid)
        return self.client.post(path, params, **options)

    def create_project(self, params=None, **options):
        """Create a project
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/projects"
        return self.client.post(path, params, **options)

    def create_project_for_team(self, team_gid, params=None, **options):
        """Create a project in a team
        :param str team_gid: (required) Globally unique identifier for the team.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/teams/{team_gid}/projects".replace("{team_gid}", team_gid)
        return self.client.post(path, params, **options)

    def create_project_for_workspace(self, workspace_gid, params=None, **options):
        """Create a project in a workspace
        :param str workspace_gid: (required) Globally unique identifier for the workspace or organization.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/workspaces/{workspace_gid}/projects".replace("{workspace_gid}", workspace_gid)
        return self.client.post(path, params, **options)

    def delete_project(self, project_gid, params=None, **options):
        """Delete a project
        :param str project_gid: (required) Globally unique identifier for the project.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/projects/{project_gid}".replace("{project_gid}", project_gid)
        return self.client.delete(path, params, **options)

    def duplicate_project(self, project_gid, params=None, **options):
        """Duplicate a project
        :param str project_gid: (required) Globally unique identifier for the project.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/projects/{project_gid}/duplicate".replace("{project_gid}", project_gid)
        return self.client.post(path, params, **options)

    def get_project(self, project_gid, params=None, **options):
        """Get a project
        :param str project_gid: (required) Globally unique identifier for the project.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/projects/{project_gid}".replace("{project_gid}", project_gid)
        return self.client.get(path, params, **options)

    def get_projects(self, params=None, **options):
        """Get multiple projects
        :param Object params: Parameters for the request
            - workspace {str}:  The workspace or organization to filter projects on.
            - team {str}:  The team to filter projects on.
            - archived {bool}:  Only return projects whose `archived` field takes on the value of this parameter.
        :param **options
            - offset {str}:  Offset token. An offset to the next page returned by the API. A pagination request will return an offset token, which can be used as an input parameter to the next request. If an offset is not passed in, the API will return the first page of results. 'Note: You can only pass in an offset that was returned to you via a previously paginated request.'
            - limit {int}:  Results per page. The number of objects to return per page. The value must be between 1 and 100.
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/projects"
        return self.client.get_collection(path, params, **options)

    def get_projects_for_task(self, task_gid, params=None, **options):
        """Get projects a task is in
        :param str task_gid: (required) The task to operate on.
        :param Object params: Parameters for the request
        :param **options
            - offset {str}:  Offset token. An offset to the next page returned by the API. A pagination request will return an offset token, which can be used as an input parameter to the next request. If an offset is not passed in, the API will return the first page of results. 'Note: You can only pass in an offset that was returned to you via a previously paginated request.'
            - limit {int}:  Results per page. The number of objects to return per page. The value must be between 1 and 100.
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/tasks/{task_gid}/projects".replace("{task_gid}", task_gid)
        return self.client.get_collection(path, params, **options)

    def get_projects_for_team(self, team_gid, params=None, **options):
        """Get a team's projects
        :param str team_gid: (required) Globally unique identifier for the team.
        :param Object params: Parameters for the request
            - archived {bool}:  Only return projects whose `archived` field takes on the value of this parameter.
        :param **options
            - offset {str}:  Offset token. An offset to the next page returned by the API. A pagination request will return an offset token, which can be used as an input parameter to the next request. If an offset is not passed in, the API will return the first page of results. 'Note: You can only pass in an offset that was returned to you via a previously paginated request.'
            - limit {int}:  Results per page. The number of objects to return per page. The value must be between 1 and 100.
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/teams/{team_gid}/projects".replace("{team_gid}", team_gid)
        return self.client.get_collection(path, params, **options)

    def get_projects_for_workspace(self, workspace_gid, params=None, **options):
        """Get all projects in a workspace
        :param str workspace_gid: (required) Globally unique identifier for the workspace or organization.
        :param Object params: Parameters for the request
            - archived {bool}:  Only return projects whose `archived` field takes on the value of this parameter.
        :param **options
            - offset {str}:  Offset token. An offset to the next page returned by the API. A pagination request will return an offset token, which can be used as an input parameter to the next request. If an offset is not passed in, the API will return the first page of results. 'Note: You can only pass in an offset that was returned to you via a previously paginated request.'
            - limit {int}:  Results per page. The number of objects to return per page. The value must be between 1 and 100.
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/workspaces/{workspace_gid}/projects".replace("{workspace_gid}", workspace_gid)
        return self.client.get_collection(path, params, **options)

    def get_task_counts_for_project(self, project_gid, params=None, **options):
        """Get task count of a project
        :param str project_gid: (required) Globally unique identifier for the project.
        :param Object params: Parameters for the request
        :param **options
            - offset {str}:  Offset token. An offset to the next page returned by the API. A pagination request will return an offset token, which can be used as an input parameter to the next request. If an offset is not passed in, the API will return the first page of results. 'Note: You can only pass in an offset that was returned to you via a previously paginated request.'
            - limit {int}:  Results per page. The number of objects to return per page. The value must be between 1 and 100.
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/projects/{project_gid}/task_counts".replace("{project_gid}", project_gid)
        return self.client.get(path, params, **options)

    def project_save_as_template(self, project_gid, params=None, **options):
        """Create a project template from a project
        :param str project_gid: (required) Globally unique identifier for the project.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/projects/{project_gid}/saveAsTemplate".replace("{project_gid}", project_gid)
        return self.client.post(path, params, **options)

    def remove_custom_field_setting_for_project(self, project_gid, params=None, **options):
        """Remove a custom field from a project
        :param str project_gid: (required) Globally unique identifier for the project.
        :param Object params: Parameters for the request
        :param **options
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/projects/{project_gid}/removeCustomFieldSetting".replace("{project_gid}", project_gid)
        return self.client.post(path, params, **options)

    def remove_followers_for_project(self, project_gid, params=None, **options):
        """Remove followers from a project
        :param str project_gid: (required) Globally unique identifier for the project.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/projects/{project_gid}/removeFollowers".replace("{project_gid}", project_gid)
        return self.client.post(path, params, **options)

    def remove_members_for_project(self, project_gid, params=None, **options):
        """Remove users from a project
        :param str project_gid: (required) Globally unique identifier for the project.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/projects/{project_gid}/removeMembers".replace("{project_gid}", project_gid)
        return self.client.post(path, params, **options)

    def update_project(self, project_gid, params=None, **options):
        """Update a project
        :param str project_gid: (required) Globally unique identifier for the project.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/projects/{project_gid}".replace("{project_gid}", project_gid)
        return self.client.put(path, params, **options)



from .helpers import *


class TestClientWebhooks(ClientTestCase):

    def setUp(self):
        ClientTestCase.setUp(self)
        self.webhook_data = {
            "id": 222,
            "resource": {
                "id": 111,
                "name": "the resource"
            },
            "target": "https://foo/123",
            "active": true
        }

    def test_webhooks_create_webhook(self):
        req = { "resource": 111, "target": "https://foo/123" }
        res = { "data": self.webhook_data }
        responses.add(POST, 'http://app/webhooks', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.webhooks.create_webhook(req), self.webhook_data)

    def test_webhooks_get_webhooks(self):
        req = { "workspace": 1337 }
        res = { "data": [self.webhook_data] }
        responses.add(GET, 'http://app/webhooks?workspace=1337', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.webhooks.get_webhooks(req), [self.webhook_data])

    def test_webhooks_get_webhook(self):
        res = { "data": self.webhook_data }
        responses.add(GET, 'http://app/webhooks/222', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.webhooks.get_webhook('222'), self.webhook_data)

    def test_webhooks_delete_webhook(self):
        res = { "data": {} }
        responses.add(DELETE, 'http://app/webhooks/222', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.webhooks.delete_webhook('222'), {})



# coding=utf-8
class _Typeahead:

    def __init__(self, client=None):
        self.client = client

    def typeahead_for_workspace(self, workspace_gid, params=None, **options):
        """Get objects via typeahead
        :param str workspace_gid: (required) Globally unique identifier for the workspace or organization.
        :param Object params: Parameters for the request
            - resource_type {str}:  (required) The type of values the typeahead should return. You can choose from one of the following: `custom_field`, `project`, `project_template`, `portfolio`, `tag`, `task`, and `user`. Note that unlike in the names of endpoints, the types listed here are in singular form (e.g. `task`). Using multiple types is not yet supported.
            - type {str}:  *Deprecated: new integrations should prefer the resource_type field.*
            - query {str}:  The string that will be used to search for relevant objects. If an empty string is passed in, the API will return results.
            - count {int}:  The number of results to return. The default is 20 if this parameter is omitted, with a minimum of 1 and a maximum of 100. If there are fewer results found than requested, all will be returned.
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/workspaces/{workspace_gid}/typeahead".replace("{workspace_gid}", workspace_gid)
        return self.client.get_collection(path, params, **options)



import json

import responses

from .helpers import ClientTestCase, GET, DELETE, POST, PUT


class TestClientOrganizationExports(ClientTestCase):
    def test_organization_exports_get_organization_export(self):
        resp = {
            "data": {
                "id": 1234,
                "created_at": '2012-02-22T02:06:58.147Z',
                "download_url":
                    'https://asana-export.s3.amazonaws.com/export-1.json.gz',
                "state": "pending",
                "organization": {
                    "id": 14916, "name": "My Workspace"
                }
            }
        }

        responses.add(
            GET, 'http://app/organization_exports/1234', status=200,
            body=json.dumps(resp), match_querystring=True)
        self.assertEqual(
            self.client.organization_exports.get_organization_export('1234'), resp['data'])

    def test_organization_exports_create_organization_export(self):
        resp = {
            "data": {
                "organization": 14916
            }
        }

        responses.add(
            POST, 'http://app/organization_exports', status=200,
            body=json.dumps(resp), match_querystring=True)
        self.assertEqual(
            self.client.organization_exports.create_organization_export(), resp['data'])




from .gen.workspaces import _Workspaces

class Workspaces(_Workspaces):
    """Workspaces resource"""
    def find_by_id(self, workspace, params={}, **options):
        """Returns the full workspace record for a single workspace.

        Parameters
        ----------
        workspace : {Id} Globally unique identifier for the workspace or organization.
        [params] : {Object} Parameters for the request
        """
        path = "/workspaces/%s" % (workspace)
        return self.client.get(path, params, **options)

    def find_all(self, params={}, **options):
        """Returns the compact records for all workspaces visible to the authorized user.

        Parameters
        ----------
        [params] : {Object} Parameters for the request
        """
        return self.client.get_collection("/workspaces", params, **options)

    def update(self, workspace, params={}, **options):
        """A specific, existing workspace can be updated by making a PUT request on
        the URL for that workspace. Only the fields provided in the data block
        will be updated; any unspecified fields will remain unchanged.

        Currently the only field that can be modified for a workspace is its `name`.

        Returns the complete, updated workspace record.

        Parameters
        ----------
        workspace : {Id} The workspace to update.
        [data] : {Object} Data for the request
        """
        path = "/workspaces/%s" % (workspace)
        return self.client.put(path, params, **options)

    def typeahead(self, workspace, params={}, **options):
        """Retrieves objects in the workspace based on an auto-completion/typeahead
        search algorithm. This feature is meant to provide results quickly, so do
        not rely on this API to provide extremely accurate search results. The
        result set is limited to a single page of results with a maximum size,
        so you won't be able to fetch large numbers of results.

        Parameters
        ----------
        workspace : {Id} The workspace to fetch objects from.
        [params] : {Object} Parameters for the request
          - resource_type : {Enum} The type of values the typeahead should return. You can choose from
          one of the following: custom_field, project, tag, task, and user.
          Note that unlike in the names of endpoints, the types listed here are
          in singular form (e.g. `task`). Using multiple types is not yet supported.
          - [type] : {Enum} **Deprecated: new integrations should prefer the resource_type field.**
          - [query] : {String} The string that will be used to search for relevant objects. If an
          empty string is passed in, the API will currently return an empty
          result set.
          - [count] : {Number} The number of results to return. The default is `20` if this
          parameter is omitted, with a minimum of `1` and a maximum of `100`.
          If there are fewer results found than requested, all will be returned.
        """
        path = "/workspaces/%s/typeahead" % (workspace)
        return self.client.get_collection(path, params, **options)

    def add_user(self, workspace, params={}, **options):
        """The user can be referenced by their globally unique user ID or their email address.
        Returns the full user record for the invited user.

        Parameters
        ----------
        workspace : {Id} The workspace or organization to invite the user to.
        [data] : {Object} Data for the request
          - user : {String} An identifier for the user. Can be one of an email address,
          the globally unique identifier for the user, or the keyword `me`
          to indicate the current user making the request.
        """
        path = "/workspaces/%s/addUser" % (workspace)
        return self.client.post(path, params, **options)

    def remove_user(self, workspace, params={}, **options):
        """The user making this call must be an admin in the workspace.
        Returns an empty data record.

        Parameters
        ----------
        workspace : {Id} The workspace or organization to invite the user to.
        [data] : {Object} Data for the request
          - user : {String} An identifier for the user. Can be one of an email address,
          the globally unique identifier for the user, or the keyword `me`
          to indicate the current user making the request.
        """
        path = "/workspaces/%s/removeUser" % (workspace)
        return self.client.post(path, params, **options)



import json

import responses

from .helpers import asana, ClientTestCase


class TestErrors(ClientTestCase):
    def test_premium_only_402(self):
        resp_json = {
            "errors": [
                {
                    "message": (
                        "Custom Fields are not available for free "
                        "users or guests.")
                }
            ]
        }
        responses.add(
            responses.GET,
            'http://app/workspaces/12345/custom_fields', status=402,
            body=json.dumps(resp_json), match_querystring=True)
        with self.assertRaises(asana.error.PremiumOnlyError):
            self.client.custom_fields.get_custom_fields_for_workspace('12345')



# coding=utf-8
class _Workspaces:

    def __init__(self, client=None):
        self.client = client

    def add_user_for_workspace(self, workspace_gid, params=None, **options):
        """Add a user to a workspace or organization
        :param str workspace_gid: (required) Globally unique identifier for the workspace or organization.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/workspaces/{workspace_gid}/addUser".replace("{workspace_gid}", workspace_gid)
        return self.client.post(path, params, **options)

    def get_workspace(self, workspace_gid, params=None, **options):
        """Get a workspace
        :param str workspace_gid: (required) Globally unique identifier for the workspace or organization.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/workspaces/{workspace_gid}".replace("{workspace_gid}", workspace_gid)
        return self.client.get(path, params, **options)

    def get_workspaces(self, params=None, **options):
        """Get multiple workspaces
        :param Object params: Parameters for the request
        :param **options
            - offset {str}:  Offset token. An offset to the next page returned by the API. A pagination request will return an offset token, which can be used as an input parameter to the next request. If an offset is not passed in, the API will return the first page of results. 'Note: You can only pass in an offset that was returned to you via a previously paginated request.'
            - limit {int}:  Results per page. The number of objects to return per page. The value must be between 1 and 100.
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/workspaces"
        return self.client.get_collection(path, params, **options)

    def remove_user_for_workspace(self, workspace_gid, params=None, **options):
        """Remove a user from a workspace or organization
        :param str workspace_gid: (required) Globally unique identifier for the workspace or organization.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/workspaces/{workspace_gid}/removeUser".replace("{workspace_gid}", workspace_gid)
        return self.client.post(path, params, **options)

    def update_workspace(self, workspace_gid, params=None, **options):
        """Update a workspace
        :param str workspace_gid: (required) Globally unique identifier for the workspace or organization.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/workspaces/{workspace_gid}".replace("{workspace_gid}", workspace_gid)
        return self.client.put(path, params, **options)




from .gen.portfolios import _Portfolios

class Portfolios(_Portfolios):
    """Portfolios resource"""

    def create(self, params={}, **options):
        """Creates a new portfolio in the given workspace with the supplied name.

        Note that portfolios created in the Asana UI may have some state
        (like the "Priority" custom field) which is automatically added to the
        portfolio when it is created. Portfolios created via our API will **not**
        be created with the same initial state to allow integrations to create
        their own starting state on a portfolio.

        Parameters
        ----------
        [data] : {Object} Data for the request
          - workspace : {Gid} The workspace or organization in which to create the portfolio.
          - name : {String} The name of the newly-created portfolio
          - [color] : {String} An optional color for the portfolio
        """
        return self.client.post("/portfolios", params, **options)

    def find_by_id(self, portfolio, params={}, **options):
        """Returns the complete record for a single portfolio.

        Parameters
        ----------
        portfolio : {Gid} The portfolio to get.
        [params] : {Object} Parameters for the request
        """
        path = "/portfolios/%s" % (portfolio)
        return self.client.get(path, params, **options)

    def update(self, portfolio, params={}, **options):
        """An existing portfolio can be updated by making a PUT request on the
        URL for that portfolio. Only the fields provided in the `data` block will be
        updated; any unspecified fields will remain unchanged.

        Returns the complete updated portfolio record.

        Parameters
        ----------
        portfolio : {Gid} The portfolio to update.
        [data] : {Object} Data for the request
        """
        path = "/portfolios/%s" % (portfolio)
        return self.client.put(path, params, **options)

    def delete(self, portfolio, params={}, **options):
        """An existing portfolio can be deleted by making a DELETE request
        on the URL for that portfolio.

        Returns an empty data record.

        Parameters
        ----------
        portfolio : {Gid} The portfolio to delete.
        """
        path = "/portfolios/%s" % (portfolio)
        return self.client.delete(path, params, **options)

    def find_all(self, params={}, **options):
        """Returns a list of the portfolios in compact representation that are owned
        by the current API user.

        Parameters
        ----------
        [params] : {Object} Parameters for the request
          - workspace : {Gid} The workspace or organization to filter portfolios on.
          - owner : {String} The user who owns the portfolio. Currently, API users can only get a
          list of portfolios that they themselves own.
        """
        return self.client.get_collection("/portfolios", params, **options)

    def get_items(self, portfolio, params={}, **options):
        """Get a list of the items in compact form in a portfolio.

        Parameters
        ----------
        portfolio : {Gid} The portfolio from which to get the list of items.
        [params] : {Object} Parameters for the request
        """
        path = "/portfolios/%s/items" % (portfolio)
        return self.client.get_collection(path, params, **options)

    def add_item(self, portfolio, params={}, **options):
        """Add an item to a portfolio.

        Returns an empty data block.

        Parameters
        ----------
        portfolio : {Gid} The portfolio to which to add an item.
        [data] : {Object} Data for the request
          - item : {Gid} The item to add to the portfolio.
          - [insert_before] : {Gid} An id of an item in this portfolio. The new item will be added before the one specified here.
          `insert_before` and `insert_after` parameters cannot both be specified.
          - [insert_after] : {Gid} An id of an item in this portfolio. The new item will be added after the one specified here.
          `insert_before` and `insert_after` parameters cannot both be specified.
        """
        path = "/portfolios/%s/addItem" % (portfolio)
        return self.client.post(path, params, **options)

    def remove_item(self, portfolio, params={}, **options):
        """Remove an item to a portfolio.

        Returns an empty data block.

        Parameters
        ----------
        portfolio : {Gid} The portfolio from which to remove the item.
        [data] : {Object} Data for the request
          - item : {Gid} The item to remove from the portfolio.
        """
        path = "/portfolios/%s/removeItem" % (portfolio)
        return self.client.post(path, params, **options)

    def add_members(self, portfolio, params={}, **options):
        """Adds the specified list of users as members of the portfolio. Returns the updated portfolio record.

        Parameters
        ----------
        portfolio : {Gid} The portfolio to add members to.
        [data] : {Object} Data for the request
          - members : {Array} An array of user ids.
        """
        path = "/portfolios/%s/addMembers" % (portfolio)
        return self.client.post(path, params, **options)

    def remove_members(self, portfolio, params={}, **options):
        """Removes the specified list of members from the portfolio. Returns the updated portfolio record.

        Parameters
        ----------
        portfolio : {Gid} The portfolio to remove members from.
        [data] : {Object} Data for the request
          - members : {Array} An array of user ids.
        """
        path = "/portfolios/%s/removeMembers" % (portfolio)
        return self.client.post(path, params, **options)

    def custom_field_settings(self, portfolio, params={}, **options):
        """Get the custom field settings on a portfolio.

        Parameters
        ----------
        portfolio : {Gid} The portfolio from which to get the custom field settings.
        [params] : {Object} Parameters for the request
        """
        path = "/portfolios/%s/custom_field_settings" % (portfolio)
        return self.client.get_collection(path, params, **options)

    def add_custom_field_setting(self, portfolio, params={}, **options):
        """Create a new custom field setting on the portfolio. Returns the full
        record for the new custom field setting.

        Parameters
        ----------
        portfolio : {Gid} The portfolio onto which to add the custom field.
        [data] : {Object} Data for the request
          - custom_field : {Gid} The id of the custom field to add to the portfolio.
          - [is_important] : {Boolean} Whether this field should be considered important to this portfolio (for instance, to display in the list view of items in the portfolio).
          - [insert_before] : {Gid} An id of a custom field setting on this portfolio. The new custom field setting will be added before this one.
          `insert_before` and `insert_after` parameters cannot both be specified.
          - [insert_after] : {Gid} An id of a custom field setting on this portfolio. The new custom field setting will be added after this one.
          `insert_before` and `insert_after` parameters cannot both be specified.
        """
        path = "/portfolios/%s/addCustomFieldSetting" % (portfolio)
        return self.client.post(path, params, **options)

    def remove_custom_field_setting(self, portfolio, params={}, **options):
        """Remove a custom field setting on the portfolio. Returns an empty data
        block.

        Parameters
        ----------
        portfolio : {Gid} The portfolio from which to remove the custom field.
        [data] : {Object} Data for the request
          - custom_field : {Gid} The id of the custom field to remove from this portfolio.
        """
        path = "/portfolios/%s/removeCustomFieldSetting" % (portfolio)
        return self.client.post(path, params, **options)




from .gen.user_task_lists import _UserTaskLists

class UserTaskLists(_UserTaskLists):
    """User Task Lists resource"""
    def find_by_user(self, user, params={}, **options):
        """Returns the full record for the user task list for the given user

        Parameters
        ----------
        user : {String} An identifier for the user. Can be one of an email address,
        the globally unique identifier for the user, or the keyword `me`
        to indicate the current user making the request.
        [params] : {Object} Parameters for the request
          - workspace : {Gid} Globally unique identifier for the workspace or organization.
        """
        path = "/users/%s/user_task_list" % (user)
        return self.client.get(path, params, **options)

    def find_by_id(self, user_task_list, params={}, **options):
        """Returns the full record for a user task list.

        Parameters
        ----------
        user_task_list : {Gid} Globally unique identifier for the user task list.
        [params] : {Object} Parameters for the request
        """
        path = "/user_task_lists/%s" % (user_task_list)
        return self.client.get(path, params, **options)

    def tasks(self, user_task_list, params={}, **options):
        """Returns the compact list of tasks in a user's My Tasks list. The returned
        tasks will be in order within each assignee status group of `Inbox`,
        `Today`, and `Upcoming`.

        **Note:** tasks in `Later` have a different ordering in the Asana web app
        than the other assignee status groups; this endpoint will still return
        them in list order in `Later` (differently than they show up in Asana,
        but the same order as in Asana's mobile apps).

        **Note:** Access control is enforced for this endpoint as with all Asana
        API endpoints, meaning a user's private tasks will be filtered out if the
        API-authenticated user does not have access to them.

        **Note:** Both complete and incomplete tasks are returned by default
        unless they are filtered out (for example, setting `completed_since=now`
        will return only incomplete tasks, which is the default view for "My
        Tasks" in Asana.)

        Parameters
        ----------
        user_task_list : {Gid} The user task list in which to search for tasks.
        [params] : {Object} Parameters for the request
          - [completed_since] : {String} Only return tasks that are either incomplete or that have been
          completed since this time.
        """
        path = "/user_task_lists/%s/tasks" % (user_task_list)
        return self.client.get_collection(path, params, **options)





from .helpers import *


class TestClientAuditLogAPI(ClientTestCase):

    def test_get_audit_log_events(self):
        res = {
            "data": [
                {
                    "actor": {
                        "actor_type": "user",
                        "email": "gregsanchez@example.com",
                        "gid": "1111",
                        "name": "Greg Sanchez"
                    },
                    "context": {
                        "api_authentication_method": "cookie",
                        "client_ip_address": "1.1.1.1",
                        "context_type": "web",
                        "oauth_app_name": "string",
                        "user_agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.103 Safari/537.36"
                    },
                    "created_at": "2021-01-01T00:00:00.000Z",
                    "details": {},
                    "event_category": "deletion",
                    "event_type": "task_deleted",
                    "gid": "12345",
                    "resource": {
                        "email": "string",
                        "gid": "1111",
                        "name": "Example Task",
                        "resource_subtype": "milestone",
                        "resource_type": "task"
                    }
                }
            ],
            "next_page": {
                "offset": "a",
                "path": "/workspaces/1337/audit_log_events?offset=a",
                "uri": "https://app.asana.com/api/1.0/workspaces/1337/audit_log_events?offset=a"
            }
        }
        responses.add(GET, 'http://app/workspaces/1337/audit_log_events?',
                      status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(
            self.client.audit_log_api.get_audit_log_events("1337"), res['data'])

    def test_get_audit_log_events_iterator(self):
        res1 = {
            "data": [
                {
                    "actor": {
                        "actor_type": "user",
                        "email": "gregsanchez@example.com",
                        "gid": "1111",
                        "name": "Greg Sanchez"
                    },
                    "context": {
                        "api_authentication_method": "cookie",
                        "client_ip_address": "1.1.1.1",
                        "context_type": "web",
                        "oauth_app_name": "string",
                        "user_agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.103 Safari/537.36"
                    },
                    "created_at": "2021-01-01T00:00:00.000Z",
                    "details": {},
                    "event_category": "deletion",
                    "event_type": "task_deleted",
                    "gid": "12345",
                    "resource": {
                        "email": "string",
                        "gid": "1111",
                        "name": "Example Task",
                        "resource_subtype": "milestone",
                        "resource_type": "task"
                    }
                }
            ],
            "next_page": {
                "offset": "a",
                "path": "/workspaces/1337/audit_log_events?limit=1&offset=a",
                "uri": "https://app.asana.com/api/1.0/workspaces/1337/audit_log_events?limit=1&offset=a"
            }
        }
        res2 = {
            "data": [
                {
                    "actor": {
                        "actor_type": "user",
                        "email": "tbizarro@example.com",
                        "gid": "1234",
                        "name": "Tim Bizarro"
                    },
                    "context": {
                        "api_authentication_method": "cookie",
                        "client_ip_address": "1.1.1.1",
                        "context_type": "web",
                        "oauth_app_name": "string",
                        "user_agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.103 Safari/537.36"
                    },
                    "created_at": "2021-01-01T00:00:00.000Z",
                    "details": {},
                    "event_category": "deletion",
                    "event_type": "task_deleted",
                    "gid": "12345",
                    "resource": {
                        "email": "string",
                        "gid": "1111",
                        "name": "Example Task",
                        "resource_subtype": "milestone",
                        "resource_type": "task"
                    }
                }
            ],
            "next_page": {
                "offset": "b",
                "path": "/workspaces/1337/audit_log_events?limit=1&offset=b",
                "uri": "https://app.asana.com/api/1.0/workspaces/1337/audit_log_events?limit=1&offset=b"
            }
        }
        res3 = {
            "data": [],
            "next_page": {
                "offset": "c",
                "path": "/workspaces/1337/audit_log_events?limit=1&offset=c",
                "uri": "https://app.asana.com/api/1.0/workspaces/1337/audit_log_events?limit=1&offset=c"
            }
        }
        res4 = {
            "data": [],
            "next_page": {
                "offset": "d",
                "path": "/workspaces/1337/audit_log_events?limit=1&offset=d",
                "uri": "https://app.asana.com/api/1.0/workspaces/1337/audit_log_events?limit=1&offset=d"
            }
        }
        responses.add(GET, 'http://app/workspaces/1337/audit_log_events?limit=1',
                      status=200, body=json.dumps(res1), match_querystring=True)
        responses.add(GET, 'http://app/workspaces/1337/audit_log_events?limit=1&offset=a',
                      status=200, body=json.dumps(res2), match_querystring=True)
        responses.add(GET, 'http://app/workspaces/1337/audit_log_events?limit=1&offset=b',
                      status=200, body=json.dumps(res3), match_querystring=True)
        responses.add(GET, 'http://app/workspaces/1337/audit_log_events?limit=1&offset=c',
                      status=200, body=json.dumps(res4), match_querystring=True)
        # Set iterator_type to 'items' to return an iterator
        iterator = self.client.audit_log_api.get_audit_log_events(
            "1337", page_size=1, iterator_type='items'
        )
        # Iterator should stop on third call
        self.assertEqual(next(iterator), res1['data'][0])
        self.assertEqual(next(iterator), res2['data'][0])
        self.assertRaises(StopIteration, next, (iterator))



from .gen.organization_exports import _OrganizationExports


class OrganizationExports(_OrganizationExports):
    """The :class:`OrganizationExports` object, which represents the resource
    of the same name in Asana's API.

    """

    def find_by_id(self, organization_export, params={}, **options):
        """Returns details of a previously-requested Organization export.

        Parameters
        ----------
        organization_export : {Gid} Globally unique identifier for the Organization export.
        [params] : {Object} Parameters for the request
        """
        path = "/organization_exports/%s" % (organization_export)
        return self.client.get(path, params, **options)

    def create(self, params={}, **options):
        """This method creates a request to export an Organization. Asana will complete the export at some
        point after you create the request.

        Parameters
        ----------
        [data] : {Object} Data for the request
          - organization : {Gid} Globally unique identifier for the workspace or organization.
        """
        return self.client.post("/organization_exports", params, **options)



# coding=utf-8
class _Portfolios:

    def __init__(self, client=None):
        self.client = client

    def add_custom_field_setting_for_portfolio(self, portfolio_gid, params=None, **options):
        """Add a custom field to a portfolio
        :param str portfolio_gid: (required) Globally unique identifier for the portfolio.
        :param Object params: Parameters for the request
        :param **options
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/portfolios/{portfolio_gid}/addCustomFieldSetting".replace("{portfolio_gid}", portfolio_gid)
        return self.client.post(path, params, **options)

    def add_item_for_portfolio(self, portfolio_gid, params=None, **options):
        """Add a portfolio item
        :param str portfolio_gid: (required) Globally unique identifier for the portfolio.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/portfolios/{portfolio_gid}/addItem".replace("{portfolio_gid}", portfolio_gid)
        return self.client.post(path, params, **options)

    def add_members_for_portfolio(self, portfolio_gid, params=None, **options):
        """Add users to a portfolio
        :param str portfolio_gid: (required) Globally unique identifier for the portfolio.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/portfolios/{portfolio_gid}/addMembers".replace("{portfolio_gid}", portfolio_gid)
        return self.client.post(path, params, **options)

    def create_portfolio(self, params=None, **options):
        """Create a portfolio
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/portfolios"
        return self.client.post(path, params, **options)

    def delete_portfolio(self, portfolio_gid, params=None, **options):
        """Delete a portfolio
        :param str portfolio_gid: (required) Globally unique identifier for the portfolio.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/portfolios/{portfolio_gid}".replace("{portfolio_gid}", portfolio_gid)
        return self.client.delete(path, params, **options)

    def get_items_for_portfolio(self, portfolio_gid, params=None, **options):
        """Get portfolio items
        :param str portfolio_gid: (required) Globally unique identifier for the portfolio.
        :param Object params: Parameters for the request
        :param **options
            - offset {str}:  Offset token. An offset to the next page returned by the API. A pagination request will return an offset token, which can be used as an input parameter to the next request. If an offset is not passed in, the API will return the first page of results. 'Note: You can only pass in an offset that was returned to you via a previously paginated request.'
            - limit {int}:  Results per page. The number of objects to return per page. The value must be between 1 and 100.
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/portfolios/{portfolio_gid}/items".replace("{portfolio_gid}", portfolio_gid)
        return self.client.get_collection(path, params, **options)

    def get_portfolio(self, portfolio_gid, params=None, **options):
        """Get a portfolio
        :param str portfolio_gid: (required) Globally unique identifier for the portfolio.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/portfolios/{portfolio_gid}".replace("{portfolio_gid}", portfolio_gid)
        return self.client.get(path, params, **options)

    def get_portfolios(self, params=None, **options):
        """Get multiple portfolios
        :param Object params: Parameters for the request
            - workspace {str}:  (required) The workspace or organization to filter portfolios on.
            - owner {str}:  The user who owns the portfolio. Currently, API users can only get a list of portfolios that they themselves own.
        :param **options
            - offset {str}:  Offset token. An offset to the next page returned by the API. A pagination request will return an offset token, which can be used as an input parameter to the next request. If an offset is not passed in, the API will return the first page of results. 'Note: You can only pass in an offset that was returned to you via a previously paginated request.'
            - limit {int}:  Results per page. The number of objects to return per page. The value must be between 1 and 100.
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/portfolios"
        return self.client.get_collection(path, params, **options)

    def remove_custom_field_setting_for_portfolio(self, portfolio_gid, params=None, **options):
        """Remove a custom field from a portfolio
        :param str portfolio_gid: (required) Globally unique identifier for the portfolio.
        :param Object params: Parameters for the request
        :param **options
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/portfolios/{portfolio_gid}/removeCustomFieldSetting".replace("{portfolio_gid}", portfolio_gid)
        return self.client.post(path, params, **options)

    def remove_item_for_portfolio(self, portfolio_gid, params=None, **options):
        """Remove a portfolio item
        :param str portfolio_gid: (required) Globally unique identifier for the portfolio.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/portfolios/{portfolio_gid}/removeItem".replace("{portfolio_gid}", portfolio_gid)
        return self.client.post(path, params, **options)

    def remove_members_for_portfolio(self, portfolio_gid, params=None, **options):
        """Remove users from a portfolio
        :param str portfolio_gid: (required) Globally unique identifier for the portfolio.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/portfolios/{portfolio_gid}/removeMembers".replace("{portfolio_gid}", portfolio_gid)
        return self.client.post(path, params, **options)

    def update_portfolio(self, portfolio_gid, params=None, **options):
        """Update a portfolio
        :param str portfolio_gid: (required) Globally unique identifier for the portfolio.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/portfolios/{portfolio_gid}".replace("{portfolio_gid}", portfolio_gid)
        return self.client.put(path, params, **options)



from .gen.sections import _Sections


class Sections(_Sections):
    """The :class:`Sections` object, which represents the resource of the same
    name in Asana's API.

    """
    def create_in_project(self, project, params={}, **options):
        """Creates a new section in a project.

        Returns the full record of the newly created section.

        Parameters
        ----------
        project : {Gid} The project to create the section in
        [data] : {Object} Data for the request
          - name : {String} The text to be displayed as the section name. This cannot be an empty string.
        """
        path = "/projects/%s/sections" % (project)
        return self.client.post(path, params, **options)

    def find_by_project(self, project, params={}, **options):
        """Returns the compact records for all sections in the specified project.

        Parameters
        ----------
        project : {Gid} The project to get sections from.
        [params] : {Object} Parameters for the request
        """
        path = "/projects/%s/sections" % (project)
        return self.client.get_collection(path, params, **options)

    def find_by_id(self, section, params={}, **options):
        """Returns the complete record for a single section.

        Parameters
        ----------
        section : {Gid} The section to get.
        [params] : {Object} Parameters for the request
        """
        path = "/sections/%s" % (section)
        return self.client.get(path, params, **options)

    def update(self, section, params={}, **options):
        """A specific, existing section can be updated by making a PUT request on
        the URL for that project. Only the fields provided in the `data` block
        will be updated; any unspecified fields will remain unchanged. (note that
        at this time, the only field that can be updated is the `name` field.)

        When using this method, it is best to specify only those fields you wish
        to change, or else you may overwrite changes made by another user since
        you last retrieved the task.

        Returns the complete updated section record.

        Parameters
        ----------
        section : {Gid} The section to update.
        [data] : {Object} Data for the request
        """
        path = "/sections/%s" % (section)
        return self.client.put(path, params, **options)

    def delete(self, section, params={}, **options):
        """A specific, existing section can be deleted by making a DELETE request
        on the URL for that section.

        Note that sections must be empty to be deleted.

        The last remaining section in a board view cannot be deleted.

        Returns an empty data block.

        Parameters
        ----------
        section : {Gid} The section to delete.
        """
        path = "/sections/%s" % (section)
        return self.client.delete(path, params, **options)

    def add_task(self, section, params={}, **options):
        """Add a task to a specific, existing section. This will remove the task from other sections of the project.

        The task will be inserted at the top of a section unless an `insert_before` or `insert_after` parameter is declared.

        This does not work for separators (tasks with the `resource_subtype` of section).

        Parameters
        ----------
        section : {Gid} The section to add a task to
        [data] : {Object} Data for the request
          - task : {Gid} The task to add to this section
          - [insert_before] : {Gid} Insert the given task immediately before the task specified by this parameter. Cannot be provided together with `insert_after`.
          - [insert_after] : {Gid} Insert the given task immediately after the task specified by this parameter. Cannot be provided together with `insert_before`.
        """
        path = "/sections/%s/addTask" % (section)
        return self.client.post(path, params, **options)

    def insert_in_project(self, project, params={}, **options):
        """Move sections relative to each other in a board view. One of
        `before_section` or `after_section` is required.

        Sections cannot be moved between projects.

        At this point in time, moving sections is not supported in list views, only board views.

        Returns an empty data block.

        Parameters
        ----------
        project : {Gid} The project in which to reorder the given section
        [data] : {Object} Data for the request
          - section : {Gid} The section to reorder
          - [before_section] : {Gid} Insert the given section immediately before the section specified by this parameter.
          - [after_section] : {Gid} Insert the given section immediately after the section specified by this parameter.
        """
        path = "/projects/%s/sections/insert" % (project)
        return self.client.post(path, params, **options)



# coding=utf-8
class _GoalRelationships:

    def __init__(self, client=None):
        self.client = client

    def add_supporting_relationship(self, goal_gid, params=None, **options):
        """Add a supporting goal relationship
        :param str goal_gid: (required) Globally unique identifier for the goal.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/goals/{goal_gid}/addSupportingRelationship".replace("{goal_gid}", goal_gid)
        return self.client.post(path, params, **options)

    def get_goal_relationship(self, goal_relationship_gid, params=None, **options):
        """Get a goal relationship
        :param str goal_relationship_gid: (required) Globally unique identifier for the goal relationship.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/goal_relationships/{goal_relationship_gid}".replace("{goal_relationship_gid}", goal_relationship_gid)
        return self.client.get(path, params, **options)

    def get_goal_relationships(self, params=None, **options):
        """Get goal relationships
        :param Object params: Parameters for the request
            - supported_goal {str}:  (required) Globally unique identifier for the supported goal in the goal relationship.
            - resource_subtype {str}:  If provided, filter to goal relationships with a given resource_subtype.
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/goal_relationships"
        return self.client.get_collection(path, params, **options)

    def remove_supporting_relationship(self, goal_gid, params=None, **options):
        """Removes a supporting goal relationship
        :param str goal_gid: (required) Globally unique identifier for the goal.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/goals/{goal_gid}/removeSupportingRelationship".replace("{goal_gid}", goal_gid)
        return self.client.post(path, params, **options)

    def update_goal_relationship(self, goal_relationship_gid, params=None, **options):
        """Update a goal relationship
        :param str goal_relationship_gid: (required) Globally unique identifier for the goal relationship.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/goal_relationships/{goal_relationship_gid}".replace("{goal_relationship_gid}", goal_relationship_gid)
        return self.client.put(path, params, **options)




from .gen.attachments import _Attachments

class Attachments(_Attachments):
    """Attachments resource"""

    def create_attachment_for_task(self, task_id, file_content, file_name, file_content_type=None, **options):
        """Upload an attachment for a task. Accepts a file object or string, file name, and optional file Content-Type"""
        path = '/tasks/%s/attachments' % (task_id)
        return self.client.request('post', path, files=[('file', (file_name, file_content, file_content_type))], **options)

    def create_on_task(self, task_id, file_content, file_name, file_content_type=None, **options):
        path = '/tasks/%s/attachments' % (task_id)
        return self.client.request('post', path, files=[('file', (file_name, file_content, file_content_type))], **options)

    def find_by_id(self, attachment, params={}, **options):
        """Returns the full record for a single attachment.

        Parameters
        ----------
        attachment : {Gid} Globally unique identifier for the attachment.
        [params] : {Object} Parameters for the request
        """
        path = "/attachments/%s" % (attachment)
        return self.client.get(path, params, **options)

    def find_by_task(self, task, params={}, **options):
        """Returns the compact records for all attachments on the task.

        Parameters
        ----------
        task : {Gid} Globally unique identifier for the task.
        [params] : {Object} Parameters for the request
        """
        path = "/tasks/%s/attachments" % (task)
        return self.client.get_collection(path, params, **options)




from .gen.users import _Users

class Users(_Users):
    """Users resource"""
    def me(self, params={}, **options):
        """Returns the full user record for the currently authenticated user.

        Parameters
        ----------
        [params] : {Object} Parameters for the request
        """
        return self.client.get("/users/me", params, **options)

    def find_by_id(self, user, params={}, **options):
        """Returns the full user record for the single user with the provided ID.

        Parameters
        ----------
        user : {String} An identifier for the user. Can be one of an email address,
        the globally unique identifier for the user, or the keyword `me`
        to indicate the current user making the request.
        [params] : {Object} Parameters for the request
        """
        path = "/users/%s" % (user)
        return self.client.get(path, params, **options)

    def get_user_favorites(self, user, params={}, **options):
        """Returns all of a user's favorites in the given workspace, of the given type.
        Results are given in order (The same order as Asana's sidebar).

        Parameters
        ----------
        user : {String} An identifier for the user. Can be one of an email address,
        the globally unique identifier for the user, or the keyword `me`
        to indicate the current user making the request.
        [params] : {Object} Parameters for the request
          - workspace : {Id} The workspace in which to get favorites.
          - resource_type : {Enum} The resource type of favorites to be returned.
        """
        path = "/users/%s/favorites" % (user)
        return self.client.get_collection(path, params, **options)

    def find_by_workspace(self, workspace, params={}, **options):
        """Returns the user records for all users in the specified workspace or
        organization.

        Parameters
        ----------
        workspace : {Id} The workspace in which to get users.
        [params] : {Object} Parameters for the request
        """
        path = "/workspaces/%s/users" % (workspace)
        return self.client.get_collection(path, params, **options)

    def find_all(self, params={}, **options):
        """Returns the user records for all users in all workspaces and organizations
        accessible to the authenticated user. Accepts an optional workspace ID
        parameter.

        Parameters
        ----------
        [params] : {Object} Parameters for the request
          - [workspace] : {Id} The workspace or organization to filter users on.
        """
        return self.client.get_collection("/users", params, **options)





from .helpers import *

class TestClientCustomFieldSettings(ClientTestCase):
    def test_custom_field_settings_get_custom_field_settings_for_project(self):
        res = {
            "data": [
                {
                    "id": 258147,
                    "custom_field": [
                        {
                            "id": 124578,
                            "name": "Priority",
                            "type": "enum"
                            }
                        ],
                    "project": [
                        {
                            "id": 1331,
                            "name": "Things to buy"
                            }
                        ]
                },
                {
                    "id": 369852,
                    "custom_field": [
                        {
                            "id": 235689,
                            "name": "Cost",
                            "type": "enum"
                            }
                        ],
                    "project": [
                        {
                            "id": 1331,
                            "name": "Things to buy"
                            }
                        ]
                },
            ]
        }
        responses.add(GET, 'http://app/projects/1331/custom_field_settings', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.custom_field_settings.get_custom_field_settings_for_project('1331'), res['data'])




from .helpers import *

class TestClientJobs(ClientTestCase):
    def test_jobs_get_job(self):
        res = {
            "data": {
                "id": 1331,
                "name": "Things to Buy"
            }
        }
        responses.add(GET, 'http://app/jobs/1331', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.jobs.get_job('1331'), res['data'])





from .gen.audit_log_api import _AuditLogAPI
from ..page_iterator import AuditLogAPIIterator


class AuditLogAPI(_AuditLogAPI):
    """AuditLogAPI resource"""

    def get_audit_log_events(self, workspace_gid, params={}, **options):
        """Override get_audit_log_events to handle non-empty next_page parameter"""
        path = "/workspaces/{workspace_gid}/audit_log_events".replace(
            "{workspace_gid}", workspace_gid)
        options = self.client._merge_options(options)
        if options['iterator_type'] == 'items':
            return AuditLogAPIIterator(self.client, path, params, options).items()
        if options['iterator_type'] is None:
            return self.client.get(path, params, **options)
        raise Exception('Unknown value for "iterator_type" option: {}'.format(
            str(options['iterator_type'])))




from .gen.jobs import _Jobs

class Jobs(_Jobs):
    """Jobs resource"""
    def find_by_id(self, job, params={}, **options):
        """Returns the complete job record for a single job.

        Parameters
        ----------
        job : {Gid} The job to get.
        [params] : {Object} Parameters for the request
        """
        path = "/jobs/%s" % (job)
        return self.client.get(path, params, **options)



from .helpers import *

class TestClientUsers(ClientTestCase):

    def test_users_me(self):
        res = {
            "data": { "email":"sanchez@...", "id": 999, "name":"Greg Sanchez" }
        }
        responses.add(GET, 'http://app/users/me', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.users.get_user('me'), res['data'])

    def test_users_get_user(self):
        res = {
            "data": {
                "email": "gsanchez@example.com",
                "id": 5678,
                "name": "Greg Sanchez",
                "workspaces": [{ "id": 1337, "name": "My Favorite Workspace" }]
            }
        }
        responses.add(GET, 'http://app/users/5678', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.users.get_user('5678'), res['data'])

    def test_users_get_users(self):
        res = {
            "data": [
                { "email": "tbizarro@example.com", "id": 1234, "name": "Tim Bizarro" },
                { "email": "gsanchez@example.com", "id": 5678, "name": "Greg Sanchez" }
            ]
        }
        # responses.add(GET, 'http://app/users?opt_fields=name,email', status=200, body=json.dumps(res), match_querystring=True)
        responses.add(GET, 'http://app/users', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.users.get_users(), res['data'])

    def test_users_get_users_for_workspace(self):
        res = {
            "data": [
                { "id": 5678, "name": "Greg Sanchez" }
            ]
        }
        responses.add(GET, 'http://app/workspaces/1337/users', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.users.get_users_for_workspace('1337'), res['data'])



from .helpers import *

class TestClientProjects(ClientTestCase):
    def test_projects_create_project(self):
        req = {
            "data": {
                "name": "Things to Buy",
                "notes": "These are things we want to purchase.",
                "workspace": 14916
            }
        }
        res = {
            "data": {
                "id": 1331,
                "name": "Things to Buy",
                "notes": "These are things we want to purchase."
            }
        }
        responses.add(POST, 'http://app/projects', status=201, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.projects.create_project(req['data']), res['data'])
        self.assertEqual(json.loads(responses.calls[0].request.body), req)

    def test_projects_get_project(self):
        res = {
            "data": {
                "id": 1331,
                "name": "Things to Buy",
                "notes": "These are things we want to purchase."
            }
        }
        responses.add(GET, 'http://app/projects/1331', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.projects.get_project('1331'), res['data'])

    def test_projects_update_project(self):
        req = {
            "data": {
                "notes": "These are things we NEED to purchase."
            }
        }
        res = {
            "data": {
                "id": 1331,
                "name": "Things to Buy",
                "notes": "These are things we NEED to purchase."
            }
        }
        responses.add(PUT, 'http://app/projects/1331', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.projects.update_project('1331', req['data']), res['data'])
        self.assertEqual(json.loads(responses.calls[0].request.body), req)

    def test_projects_delete_project(self):
        res = { "data": {} }
        responses.add(DELETE, 'http://app/projects/1331', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.projects.delete_project('1331'), res['data'])

    def test_projects_get_projects_for_workspace(self):
        res = {
            "data": [
                { "id": 1331, "name": "Things to buy" },
                { "id": 14641, "name": "Cat Stuff" }
            ]
        }
        responses.add(GET, 'http://app/workspaces/14916/projects', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.projects.get_projects_for_workspace('14916'), res['data'])

    def test_projects_get_projects(self):
        res = {
            "data": [
                { "id": 1331, "name": "Things to buy" },
                { "id": 14641, "name": "Cat Stuff" }
            ]
        }
        responses.add(GET, 'http://app/projects', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.projects.get_projects(), res['data'])

    def test_projects_add_custom_field_setting_for_project(self):
        req = {
            "data": {
                "custom_field":124578,
                "is_important": true
            }
        }
        res = {
            "data": {}
        }

        responses.add(POST, 'http://app/projects/1331/addCustomFieldSetting', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.projects.add_custom_field_setting_for_project('1331', req['data']), res['data'])
        self.assertEqual(json.loads(responses.calls[0].request.body), req)


    def test_projects_remove_custom_field_setting_for_project(self):
        req = {
            "data": {
                "custom_field":124578,
            }
        }
        res = {
            "data": {}
        }

        responses.add(POST, 'http://app/projects/1331/removeCustomFieldSetting', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.projects.remove_custom_field_setting_for_project('1331', req['data']), res['data'])
        self.assertEqual(json.loads(responses.calls[0].request.body), req)



import warnings
try:
    from mock import patch, call
except ImportError:
    from unittest.mock import patch, call

from .helpers import *

class TestClient(ClientTestCase):

    def test_version_values(self):
        values = self.client._version_values()
        self.assertEqual('Python', values['language'])
        self.assertRegex(values['version'], r'[0-9]+[.][0-9]+[.][0-9]+')
        self.assertSetEqual(
            {'language', 'version', 'language_version', 'os', 'os_version'},
            set(values.keys()))

    def test_version_header(self):
        self.assertRegex(
            self.client._version_header(), r'language=Python')

    def test_default_headers(self):
        self.client.headers['key'] = 'value'
        responses.add(GET, 'http://app/users/me', status=200, body='{"data":{}}')
        self.client.users.get_user('me')
        self.assertLessEqual(
            set({'key': 'value'}.items()),
            set(responses.calls[0].request.headers.items())
        )

    def test_request_headers(self):
        responses.add(GET, 'http://app/users/me', status=200, body='{"data":{}}')
        self.client.users.get_user('me', headers={'key': 'value'})
        self.assertLessEqual(
            set({'key': 'value'}.items()),
            set(responses.calls[0].request.headers.items())
        )

    def test_overriding_headers(self):
        self.client.headers['key'] = 'value'
        self.client.headers['key2'] = 'value2'
        responses.add(GET, 'http://app/users/me', status=200, body='{"data":{}}')
        self.client.users.get_user('me', headers={'key2': 'value3'})
        self.assertLessEqual(
            set({'key': 'value', 'key2': 'value3'}.items()),
            set(responses.calls[0].request.headers.items())
        )

    def test_content_type_headers(self):
        self.client.headers['key'] = 'value'
        responses.add(POST, 'http://app/tasks/123/addProject', status=200,
                      body='{"data":{}}')
        self.client.tasks.add_project(123, project=456, headers={'key2': 'value2'})
        self.assertLessEqual(
            set({'content-type': 'application/json'}.items()),
            set(responses.calls[0].request.headers.items())
        )

    def test_users_me_not_authorized(self):
        res = {
            "errors": [
                { "message": "Not Authorized" }
            ]
        }
        responses.add(GET, 'http://app/users/me', status=401, body=json.dumps(res), match_querystring=True)
        self.assertRaises(asana.error.NoAuthorizationError, self.client.users.get_user, 'me')

    def test_tasks_get_tasks_invalid_request(self):
        res = {
            "errors": [
                { "message": "workspace: Missing input" }
            ]
        }
        responses.add(GET, 'http://app/tasks', status=400, body=json.dumps(res), match_querystring=True)
        self.assertRaises(asana.error.InvalidRequestError, self.client.tasks.get_tasks)

    def test_users_me_server_error(self):
        res = {
            "errors": [
                {
                    "message": "Server Error",
                    "phrase": "6 sad squid snuggle softly"
                }
            ]
        }
        responses.add(GET, 'http://app/users/me', status=500, body=json.dumps(res), match_querystring=True)
        self.assertRaises(asana.error.ServerError, self.client.users.get_user, 'me', max_retries=0)

    def test_users_me_unfriendly_server_error(self):
        res = "Crappy Response"
        responses.add(GET, 'http://app/users/me', status=504, body=res, match_querystring=True)
        self.assertRaises(asana.error.ServerError, self.client.users.get_user, 'me', max_retries=0)

    def test_users_get_user_not_found(self):
        res = {
            "errors": [
                { "message": "user: Unknown object: 1234124312341" }
            ]
        }
        responses.add(GET, 'http://app/users/1234', status=404, body=json.dumps(res), match_querystring=True)
        self.assertRaises(asana.error.NotFoundError, self.client.users.get_user, ('1234'))

    def test_users_get_user_forbidden(self):
        res = {
            "errors": [
                { "message": "user: Forbidden" }
            ]
        }
        responses.add(GET, 'http://app/users/1234', status=403, body=json.dumps(res), match_querystring=True)
        self.assertRaises(asana.error.ForbiddenError, self.client.users.get_user, ('1234'))

    def test_option_pretty(self):
        res = {
            "data": { "email": "sanchez@...", "id": 999, "name": "Greg Sanchez" }
        }
        responses.add(GET, 'http://app/users/me?opt_pretty=true', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.users.get_user('me', pretty=True), res['data'])

    def test_option_fields(self):
        res = {
            "data": { "name": "Make a list", "notes": "Check it twice!", "id": 1224 }
        }
        responses.add(GET, 'http://app/tasks/1224?opt_fields=name%2Cnotes', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.tasks.get_task('1224', fields=['name','notes']), res['data'])

    def test_option_expand(self):
        req = {
            'data': { 'assignee': 1234 },
            'options': { 'expand' : ['projects'] }
        }
        res = {
            "data": {
                "id": 1001,
                "projects": [
                    {
                        "archived": false,
                        "created_at": "",
                        "followers": [],
                        "modified_at": "",
                        "notes": "",
                        "id": 1331,
                        "name": "Things to buy"
                    }
                ]
                # ...
            }
        }
        responses.add(PUT, 'http://app/tasks/1001', status=200, body=json.dumps(res), match_querystring=True)
        # -d "assignee=1234" \
        # -d "options.expand=%2Aprojects%2A"
        self.assertEqual(self.client.tasks.update_task('1001', req['data'], expand=['projects']), res['data'])
        self.assertEqual(json.loads(responses.calls[0].request.body), req)

    def test_pagination(self):
        res = {
            "data": [
                { "id": 1000, "name": "Task 1" }
            ],
            "next_page": {
                "offset": "yJ0eXAiOiJKV1QiLCJhbGciOiJIRzI1NiJ9",
                "path": "/tasks?project=1337&limit=5&offset=yJ0eXAiOiJKV1QiLCJhbGciOiJIRzI1NiJ9",
                "uri": "https://app.asana.com/api/1.0/tasks?project=1337&limit=5&offset=yJ0eXAiOiJKV1QiLCJhbGciOiJIRzI1NiJ9"
            }
        }
        responses.add(GET, 'http://app/projects/1337/tasks?limit=5&offset=eyJ0eXAiOJiKV1iQLCJhbGciOiJIUzI1NiJ9', status=200, body=json.dumps(res), match_querystring=True)

        self.assertEqual(self.client.tasks.get_tasks_for_project('1337', { 'limit': 5, 'offset': 'eyJ0eXAiOJiKV1iQLCJhbGciOiJIUzI1NiJ9'}), res['data'])

    @unittest.skip("iterator_type='pages' disabled")
    def test_page_iterator_item_limit_lt_items(self):
        responses.add(GET, 'http://app/projects/1337/tasks?limit=2', status=200, body=json.dumps({ 'data': ['a', 'b'], 'next_page': { 'offset': 'a', 'path': '/projects/1337/tasks?limit=2&offset=a' } }), match_querystring=True)
        responses.add(GET, 'http://app/projects/1337/tasks?limit=2&offset=a', status=200, body=json.dumps({ 'data': ['c'], 'next_page': null }), match_querystring=True)

        iterator = self.client.tasks.get_tasks_for_project('1337', item_limit=2, page_size=2, iterator_type='pages')
        self.assertEqual(next(iterator), ['a', 'b'])
        self.assertRaises(StopIteration, next, (iterator))

    @unittest.skip("iterator_type='pages' disabled")
    def test_page_iterator_item_limit_eq_items(self):
        responses.add(GET, 'http://app/projects/1337/tasks?limit=2', status=200, body=json.dumps({ 'data': ['a', 'b'], 'next_page': { 'offset': 'a', 'path': '/projects/1337/tasks?limit=2&offset=a' } }), match_querystring=True)
        responses.add(GET, 'http://app/projects/1337/tasks?limit=1&offset=a', status=200, body=json.dumps({ 'data': ['c'], 'next_page': null }), match_querystring=True)

        iterator = self.client.tasks.get_tasks_for_project('1337', item_limit=3, page_size=2, iterator_type='pages')
        self.assertEqual(next(iterator), ['a', 'b'])
        self.assertEqual(next(iterator), ['c'])
        self.assertRaises(StopIteration, next, (iterator))

    @unittest.skip("iterator_type='pages' disabled")
    def test_page_iterator_item_limit_gt_items(self):
        responses.add(GET, 'http://app/projects/1337/tasks?limit=2', status=200, body=json.dumps({ 'data': ['a', 'b'], 'next_page': { 'offset': 'a', 'path': '/projects/1337/tasks?limit=2&offset=a' } }), match_querystring=True)
        responses.add(GET, 'http://app/projects/1337/tasks?limit=2&offset=a', status=200, body=json.dumps({ 'data': ['c'], 'next_page': null }), match_querystring=True)

        iterator = self.client.tasks.get_tasks_for_project('1337', item_limit=4, page_size=2, iterator_type='pages')
        self.assertEqual(next(iterator), ['a', 'b'])
        self.assertEqual(next(iterator), ['c'])
        self.assertRaises(StopIteration, next, (iterator))

    def test_item_iterator_item_limit_lt_items(self):
        responses.add(GET, 'http://app/projects/1337/tasks?limit=2', status=200, body=json.dumps({ 'data': ['a', 'b'], 'next_page': { 'offset': 'a', 'path': '/projects/1337/tasks?limit=2&offset=a' } }), match_querystring=True)

        iterator = self.client.tasks.get_tasks_for_project('1337', item_limit=2, page_size=2, iterator_type='items')
        self.assertEqual(next(iterator), 'a')
        self.assertEqual(next(iterator), 'b')
        self.assertRaises(StopIteration, next, (iterator))

    def test_item_iterator_item_limit_eq_items(self):
        responses.add(GET, 'http://app/projects/1337/tasks?limit=2', status=200, body=json.dumps({ 'data': ['a', 'b'], 'next_page': { 'offset': 'a', 'path': '/projects/1337/tasks?limit=2&offset=a' } }), match_querystring=True)
        responses.add(GET, 'http://app/projects/1337/tasks?limit=1&offset=a', status=200, body=json.dumps({ 'data': ['c'], 'next_page': null }), match_querystring=True)

        iterator = self.client.tasks.get_tasks_for_project('1337', item_limit=3, page_size=2, iterator_type='items')
        self.assertEqual(next(iterator), 'a')
        self.assertEqual(next(iterator), 'b')
        self.assertEqual(next(iterator), 'c')
        self.assertRaises(StopIteration, next, (iterator))

    def test_item_iterator_item_limit_gt_items(self):
        responses.add(GET, 'http://app/projects/1337/tasks?limit=2', status=200, body=json.dumps({ 'data': ['a', 'b'], 'next_page': { 'offset': 'a', 'path': '/projects/1337/tasks?limit=2&offset=a' } }), match_querystring=True)
        responses.add(GET, 'http://app/projects/1337/tasks?limit=2&offset=a', status=200, body=json.dumps({ 'data': ['c'], 'next_page': null }), match_querystring=True)

        iterator = self.client.tasks.get_tasks_for_project('1337', item_limit=4, page_size=2, iterator_type='items')
        self.assertEqual(next(iterator), 'a')
        self.assertEqual(next(iterator), 'b')
        self.assertEqual(next(iterator), 'c')
        self.assertRaises(StopIteration, next, (iterator))

    def test_item_iterator_preserve_opt_fields(self):
        responses.add(GET, 'http://app/projects/1337/tasks?limit=2&opt_fields=foo', status=200, body=json.dumps({ 'data': ['a', 'b'], 'next_page': { 'offset': 'a', 'path': '/projects/1337/tasks?limit=2&offset=a' } }), match_querystring=True)
        responses.add(GET, 'http://app/projects/1337/tasks?limit=1&opt_fields=foo&offset=a', status=200, body=json.dumps({ 'data': ['c'], 'next_page': null }), match_querystring=True)

        iterator = self.client.tasks.get_tasks_for_project('1337', item_limit=3, page_size=2, fields=['foo'], iterator_type='items')
        self.assertEqual(next(iterator), 'a')
        self.assertEqual(next(iterator), 'b')
        self.assertEqual(next(iterator), 'c')
        self.assertRaises(StopIteration, next, (iterator))

    @patch('time.sleep')
    def test_rate_limiting(self, time_sleep):
        res = [
            (429, { 'Retry-After': '10' }, '{}'),
            (200, {}, json.dumps({ 'data': 'me' }))
        ]
        responses.add_callback(responses.GET, 'http://app/users/me', callback=lambda r: res.pop(0), content_type='application/json')

        self.assertEqual(self.client.users.get_user('me'), 'me')
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(time_sleep.mock_calls, [call(10.0)])

    @patch('time.sleep')
    def test_rate_limited_twice(self, time_sleep):
        res = [
            (429, { 'Retry-After': '10' }, '{}'),
            (429, { 'Retry-After': '1' }, '{}'),
            (200, {}, json.dumps({ 'data': 'me' }))
        ]
        responses.add_callback(responses.GET, 'http://app/users/me', callback=lambda r: res.pop(0), content_type='application/json')

        self.assertEqual(self.client.users.get_user('me'), 'me')
        self.assertEqual(len(responses.calls), 3)
        self.assertEqual(time_sleep.mock_calls, [call(10.0), call(1.0)])

    @patch('time.sleep')
    def test_server_error_retry(self, time_sleep):
        res = [
            (500, {}, '{}'),
            (500, {}, '{}'),
            (500, {}, '{}'),
            (200, {}, json.dumps({ 'data': 'me' }))
        ]
        responses.add_callback(responses.GET, 'http://app/users/me', callback=lambda r: res.pop(0), content_type='application/json')

        self.assertRaises(asana.error.ServerError, self.client.users.get_user, 'me', max_retries=2)
        self.assertEqual(time_sleep.mock_calls, [call(1.0), call(2.0)])

    @patch('time.sleep')
    def test_server_error_retry_backoff(self, time_sleep):
        res = [
            (500, {}, '{}'),
            (500, {}, '{}'),
            (500, {}, '{}'),
            (200, {}, json.dumps({ 'data': 'me' }))
        ]
        responses.add_callback(responses.GET, 'http://app/users/me', callback=lambda r: res.pop(0), content_type='application/json')

        self.assertEqual(self.client.users.get_user('me'), 'me')
        self.assertEqual(time_sleep.mock_calls, [call(1.0), call(2.0), call(4.0)])


    def test_get_named_parameters(self):
        responses.add(GET, 'http://app/tasks?workspace=14916&assignee=me', status=200, body=json.dumps({ 'data': 'dummy data' }), match_querystring=True)

        self.assertEqual(self.client.tasks.get_tasks(workspace=14916, assignee="me"), 'dummy data')

    def test_post_named_parameters(self):
        responses.add(POST, 'http://app/tasks', status=201, body='{ "data": "dummy data" }', match_querystring=True)

        self.assertEqual(self.client.tasks.create_task(assignee=1235, followers=[5678], name="Hello, world."), 'dummy data')
        self.assertEqual(json.loads(responses.calls[0].request.body), {
            "data": {
                "assignee": 1235,
                "followers": [5678],
                "name": "Hello, world."
            }
        })

    def test_put_named_parameters(self):
        responses.add(PUT, 'http://app/tasks/1001', status=200, body='{ "data": "dummy data" }', match_querystring=True)

        self.assertEqual(self.client.tasks.update_task('1001', assignee=1235, followers=[5678], name="Hello, world."), 'dummy data')
        self.assertEqual(json.loads(responses.calls[0].request.body), {
            "data": {
                "assignee": 1235,
                "followers": [5678],
                "name": "Hello, world."
            }
        })

    def test_asana_change_header_logging(self):
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")

            requestHeaders = {}
            responseHeaders = {
                'asana-change': 'name=string_ids;info=something;affected=true'
            }
            self.client._log_asana_change_header(requestHeaders, responseHeaders)

            assert len(w) == 1

    def test_asana_change_header_logging_turned_off(self):
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")

            requestHeaders = {}
            responseHeaders = {
                'asana-change': 'name=string_ids;info=something;affected=true'
            }
            self.client.LOG_ASANA_CHANGE_WARNINGS = false
            self.client._log_asana_change_header(requestHeaders, responseHeaders)

            assert len(w) == 0

    def test_asana_change_header_ignore_case(self):
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")

            requestHeaders = {}
            responseHeaders = {
                'asANa-chANge': 'name=string_ids;info=something;affected=true'
            }
            self.client._log_asana_change_header(requestHeaders, responseHeaders)

            assert len(w) == 1

    def test_asana_change_header_enable(self):
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")

            requestHeaders = {
                'asana-enable': 'string_ids'
            }
            responseHeaders = {
                'asana-change': 'name=string_ids;info=something;affected=true'
            }
            self.client._log_asana_change_header(requestHeaders, responseHeaders)

            assert len(w) == 0

    def test_asana_change_header_disable(self):
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")

            requestHeaders = {
                'asana-disable': 'string_ids'
            }
            responseHeaders = {
                'asana-change': 'name=string_ids;info=something;affected=true'
            }
            self.client._log_asana_change_header(requestHeaders, responseHeaders)

            assert len(w) == 0

    def test_asana_change_header_multiple(self):
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")

            requestHeaders = {}
            responseHeaders = {
                'asana-change': 'name=string_ids;info=something;affected=true,name=new_sections;info=something;affected=true'
            }
            self.client._log_asana_change_header(requestHeaders, responseHeaders)

            assert len(w) == 2

    def test_asana_change_header_multiple_disable(self):
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")

            requestHeaders = {
                'asana-disable': 'string_ids,new_sections'
            }
            responseHeaders = {
                'asana-change': 'name=string_ids;info=something;affected=true,name=new_sections;info=something;affected=true'
            }
            self.client._log_asana_change_header(requestHeaders, responseHeaders)

            assert len(w) == 0



import json

import responses

from .helpers import ClientTestCase, GET, DELETE, POST, PUT


class TestClientSections(ClientTestCase):

    def test_sections_create_section_for_project(self):
        resp = {
            "data": {
                "id": 2001,
                "name": "High Priority:",
                "created_at": "2017-01-17T22:06:39.249Z",
                "project": {
                    "id": 1331,
                    "name": "Things to buy"
                }
            }
        }

        responses.add(
            POST, 'http://app/projects/1331/sections', status=200,
            body=json.dumps(resp), match_querystring=True)
        self.assertEqual(
            self.client.sections.create_section_for_project('1331'), resp['data'])

    def test_sections_get_sections_for_project(self):
        resp = {
            "data": [
                {
                    "id": 2001,
                    "name": "High Priority:"
                },
                {
                    "id": 2002,
                    "name": "Low Priority:"
                },
            ]
        }

        responses.add(
            GET, 'http://app/projects/1331/sections', status=200,
            body=json.dumps(resp), match_querystring=True)
        self.assertEqual(
            self.client.sections.get_sections_for_project('1331'), resp['data'])

    def test_sections_get_section(self):
        resp = {
            "data": {
                "id": 2001,
                "name": "High Priority:",
                "created_at": "2017-01-17T22:06:39.249Z",
                "project": {
                    "id": 1331,
                    "name": "Things to buy"
                }
            }
        }

        responses.add(
            GET, 'http://app/sections/2001', status=200,
            body=json.dumps(resp), match_querystring=True)
        self.assertEqual(
            self.client.sections.get_section('2001'), resp['data'])

    def test_sections_update_section(self):
        req = {
            "name": "High Priority, renamed:"
        }
        resp = {
            "data": {
                "id": 2001,
                "name": "High Priority, renamed:",
                "created_at": "2017-01-17T22:06:39.249Z",
                "project": {
                    "id": 1331,
                    "name": "Things to buy"
                }
            }
        }

        responses.add(
            PUT, 'http://app/sections/2001', status=200,
            body=json.dumps(resp), match_querystring=True)
        self.assertEqual(
            self.client.sections.update_section('2001', req), resp['data'])

    def test_sections_delete_section(self):
        resp = {
            "data": {}
        }

        responses.add(
            DELETE, 'http://app/sections/2001', status=200,
            body=json.dumps(resp), match_querystring=True)
        self.assertEqual(
            self.client.sections.delete_section('2001'), resp['data'])

    def test_sections_insert_section_for_project(self):
        resp = {
            "data": {
                "section": 2001,
                "project": 1331,
                "before_section": 1330
            }
        }

        responses.add(
            POST, 'http://app/projects/1331/sections/insert', status=200,
            body=json.dumps(resp), match_querystring=True)
        self.assertEqual(
            self.client.sections.insert_section_for_project('1331'), resp['data'])



from .helpers import *

class TestClientPortfolioMemberships(ClientTestCase):
    def test_portfolio_memberships_get_portfolio_membership(self):
        res = {
            "data": {
                "id": 1331,
                "name": "Things to Buy"
            }
        }
        responses.add(GET, 'http://app/portfolio_memberships/1331', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.portfolio_memberships.get_portfolio_membership('1331'), res['data'])

    def test_portfolio_memberships_get_portfolio_memberships_for_portfolio(self):
        res = {
            "data": [
                {
                    "id": 1331,
                    "name": "Things to Buy"
                }
            ]
        }
        responses.add(GET, 'http://app/portfolios/1331/portfolio_memberships', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.portfolio_memberships.get_portfolio_memberships_for_portfolio('1331'), res['data'])

    def test_portfolio_memberships_get_portfolio_memberships(self):
        res = {
            "data": [
                { "id": 1331, "name": "Things to buy" },
                { "id": 14641, "name": "Cat Stuff" }
            ]
        }
        responses.add(GET, 'http://app/portfolio_memberships', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.portfolio_memberships.get_portfolio_memberships(), res['data'])



# coding=utf-8
class _Events:

    def __init__(self, client=None):
        self.client = client

    def get_events(self, params=None, **options):
        """Get events on a resource
        :param Object params: Parameters for the request
            - resource {str}:  (required) A resource ID to subscribe to. The resource can be a task or project.
            - sync {str}:  A sync token received from the last request, or none on first sync. Events will be returned from the point in time that the sync token was generated. *Note: On your first request, omit the sync token. The response will be the same as for an expired sync token, and will include a new valid sync token.If the sync token is too old (which may happen from time to time) the API will return a `412 Precondition Failed` error, and include a fresh sync token in the response.*
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/events"
        return self.client.get_collection(path, params, **options)



# coding=utf-8
class _Memberships:

    def __init__(self, client=None):
        self.client = client

    def create_membership(self, params=None, **options):
        """Create a membership
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/memberships"
        return self.client.post(path, params, **options)

    def delete_membership(self, membership_gid, params=None, **options):
        """Delete a membership
        :param str membership_gid: (required) Globally unique identifier for the membership.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/memberships/{membership_gid}".replace("{membership_gid}", membership_gid)
        return self.client.delete(path, params, **options)

    def get_memberships(self, params=None, **options):
        """Get multiple memberships
        :param Object params: Parameters for the request
            - parent {str}:  Globally unique identifier for `project`, `portfolio`,   `team`, `goal`, and `workspace`.
            - member {str}:  Globally unique identifier for `team` or `user`.
            - resource_subtype {str}:  The resource_subtype to filter on. Must be provided with `member` and `workspace` if `parent` is not provided. Valid values include `team_membership`, `workspace_membership`, `portfolio_membership`
            - workspace {str}:  The workspace to filter on. Must be provided with `member` and `resource_subtype` if `parent` is not provided.
        :param **options
            - offset {str}:  Offset token. An offset to the next page returned by the API. A pagination request will return an offset token, which can be used as an input parameter to the next request. If an offset is not passed in, the API will return the first page of results. 'Note: You can only pass in an offset that was returned to you via a previously paginated request.'
            - limit {int}:  Pagination limit for the request.
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/memberships"
        return self.client.get_collection(path, params, **options)

    def update_membership(self, membership_gid, params=None, **options):
        """Update a membership
        :param str membership_gid: (required) Globally unique identifier for the membership.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/memberships/{membership_gid}".replace("{membership_gid}", membership_gid)
        return self.client.put(path, params, **options)




from .gen.webhooks import _Webhooks

class Webhooks(_Webhooks):
    """Webhooks resource"""
    def create(self, params={}, **options):
        """Establishing a webhook is a two-part process. First, a simple HTTP POST
        similar to any other resource creation. Since you could have multiple
        webhooks we recommend specifying a unique local id for each target.

        Next comes the confirmation handshake. When a webhook is created, we will
        send a test POST to the `target` with an `X-Hook-Secret` header as
        described in the
        [Resthooks Security documentation](http://resthooks.org/docs/security/).
        The target must respond with a `200 OK` and a matching `X-Hook-Secret`
        header to confirm that this webhook subscription is indeed expected.

        If you do not acknowledge the webhook's confirmation handshake it will
        fail to setup, and you will receive an error in response to your attempt
        to create it. This means you need to be able to receive and complete the
        webhook *while* the POST request is in-flight.

        Parameters
        ----------
        resource : {Id} A resource ID to subscribe to. The resource can be a task or project.
        target : {String} The URL to receive the HTTP POST.
        [data] : {Object} Data for the request
        """
        return self.client.post("/webhooks", params, **options)

    def get_all(self, params={}, **options):
        """Returns the compact representation of all webhooks your app has
        registered for the authenticated user in the given workspace.

        Parameters
        ----------
        workspace : {Id} The workspace to query for webhooks in.
        [params] : {Object} Parameters for the request
          - [resource] : {Id} Only return webhooks for the given resource.
        """
        return self.client.get_collection("/webhooks", params, **options)

    def get_by_id(self, webhook, params={}, **options):
        """Returns the full record for the given webhook.

        Parameters
        ----------
        webhook : {Id} The webhook to get.
        [params] : {Object} Parameters for the request
        """
        path = "/webhooks/%s" % (webhook)
        return self.client.get(path, params, **options)

    def delete_by_id(self, webhook, params={}, **options):
        """This method permanently removes a webhook. Note that it may be possible
        to receive a request that was already in flight after deleting the
        webhook, but no further requests will be issued.

        Parameters
        ----------
        webhook : {Id} The webhook to delete.
        """
        path = "/webhooks/%s" % (webhook)
        return self.client.delete(path, params, **options)



from .helpers import *

class TestClientUserTaskLists(ClientTestCase):
    def test_user_task_lists_get_user_task_list(self):
        res = {
            "data": {
                "id": 1331,
                "name": "Things to Buy"
            }
        }
        responses.add(GET, 'http://app/user_task_lists/1331', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.user_task_lists.get_user_task_list('1331'), res['data'])

    def test_user_task_lists_get_user_task_list_for_user(self):
        res = {
            "data": {
                "id": 1331,
                "name": "Things to Buy"
            }
        }
        responses.add(GET, 'http://app/users/1331/user_task_list', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.user_task_lists.get_user_task_list_for_user('1331'), res['data'])

    def test_task_get_tasks_for_user_task_list(self):
        res = {
            "data": [
                { "id": 1331, "name": "Things to buy" },
                { "id": 14641, "name": "Cat Stuff" }
            ]
        }
        responses.add(GET, 'http://app/user_task_lists/1335/tasks', status=200, body=json.dumps(res), match_querystring=True)
        self.assertEqual(self.client.tasks.get_tasks_for_user_task_list('1335'), res['data'])




# coding=utf-8
class _AuditLogAPI:

    def __init__(self, client=None):
        self.client = client

    def get_audit_log_events(self, workspace_gid, params=None, **options):
        """Get audit log events
        :param str workspace_gid: (required) Globally unique identifier for the workspace or organization.
        :param Object params: Parameters for the request
            - start_at {datetime}:  Filter to events created after this time (inclusive).
            - end_at {datetime}:  Filter to events created before this time (exclusive).
            - event_type {str}:  Filter to events of this type. Refer to the [supported audit log events](/docs/audit-log-events#supported-audit-log-events) for a full list of values.
            - actor_type {str}:  Filter to events with an actor of this type. This only needs to be included if querying for actor types without an ID. If `actor_gid` is included, this should be excluded.
            - actor_gid {str}:  Filter to events triggered by the actor with this ID.
            - resource_gid {str}:  Filter to events with this resource ID.
        :param **options
            - offset {str}:  Offset token. An offset to the next page returned by the API. A pagination request will return an offset token, which can be used as an input parameter to the next request. If an offset is not passed in, the API will return the first page of results. 'Note: You can only pass in an offset that was returned to you via a previously paginated request.'
            - limit {int}:  Results per page. The number of objects to return per page. The value must be between 1 and 100.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/workspaces/{workspace_gid}/audit_log_events".replace("{workspace_gid}", workspace_gid)
        return self.client.get_collection(path, params, **options)




from .gen.custom_field_settings import _CustomFieldSettings

class CustomFieldSettings(_CustomFieldSettings):
    """Custom Field Settings resource"""
    def find_by_project(self, project, params={}, **options):
        """Returns a list of all of the custom fields settings on a project.

        Parameters
        ----------
        project : {Gid} The ID of the project for which to list custom field settings
        [params] : {Object} Parameters for the request
        """
        path = "/projects/%s/custom_field_settings" % (project)
        return self.client.get_collection(path, params, **options)

    def find_by_portfolio(self, portfolio, params={}, **options):
        """Returns a list of all of the custom fields settings on a portfolio.

        Parameters
        ----------
        portfolio : {Gid} The ID of the portfolio for which to list custom field settings
        [params] : {Object} Parameters for the request
        """
        path = "/portfolios/%s/custom_field_settings" % (portfolio)
        return self.client.get_collection(path, params, **options)




from .gen.portfolio_memberships import _PortfolioMemberships

class PortfolioMemberships(_PortfolioMemberships):
    """Portfolio Memberships resource"""

    def find_all(self, params={}, **options):
        """Returns the compact portfolio membership records for the portfolio. You must
        specify `portfolio`, `portfolio` and `user`, or `workspace` and `user`.

        Parameters
        ----------
        [params] : {Object} Parameters for the request
          - [portfolio] : {Gid} The portfolio for which to fetch memberships.
          - [workspace] : {Gid} The workspace for which to fetch memberships.
          - [user] : {String} The user to filter the memberships to.
        """
        return self.client.get_collection("/portfolio_memberships", params, **options)

    def find_by_portfolio(self, portfolio, params={}, **options):
        """Returns the compact portfolio membership records for the portfolio.

        Parameters
        ----------
        portfolio : {Gid} The portfolio for which to fetch memberships.
        [params] : {Object} Parameters for the request
          - [user] : {String} If present, the user to filter the memberships to.
        """
        path = "/portfolios/%s/portfolio_memberships" % (portfolio)
        return self.client.get_collection(path, params, **options)

    def find_by_id(self, portfolio_membership, params={}, **options):
        """Returns the portfolio membership record.

        Parameters
        ----------
        portfolio_membership : {Gid} Globally unique identifier for the portfolio membership.
        [params] : {Object} Parameters for the request
        """
        path = "/portfolio_memberships/%s" % (portfolio_membership)
        return self.client.get(path, params, **options)




# coding=utf-8
class _UserTaskLists:

    def __init__(self, client=None):
        self.client = client

    def get_user_task_list(self, user_task_list_gid, params=None, **options):
        """Get a user task list
        :param str user_task_list_gid: (required) Globally unique identifier for the user task list.
        :param Object params: Parameters for the request
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/user_task_lists/{user_task_list_gid}".replace("{user_task_list_gid}", user_task_list_gid)
        return self.client.get(path, params, **options)

    def get_user_task_list_for_user(self, user_gid, params=None, **options):
        """Get a user's task list
        :param str user_gid: (required) A string identifying a user. This can either be the string \"me\", an email, or the gid of a user.
        :param Object params: Parameters for the request
            - workspace {str}:  (required) The workspace in which to get the user task list.
        :param **options
            - opt_fields {list[str]}:  Defines fields to return. Some requests return *compact* representations of objects in order to conserve resources and complete the request more efficiently. Other times requests return more information than you may need. This option allows you to list the exact set of fields that the API should be sure to return for the objects. The field names should be provided as paths, described below. The id of included objects will always be returned, regardless of the field options.
            - opt_pretty {bool}:  Provides “pretty” output. Provides the response in a “pretty” format. In the case of JSON this means doing proper line breaking and indentation to make it readable. This will take extra time and increase the response size so it is advisable only to use this during debugging.
        :return: Object
        """
        if params is None:
            params = {}
        path = "/users/{user_gid}/user_task_list".replace("{user_gid}", user_gid)
        return self.client.get(path, params, **options)
